//  bt.h
//
//  Bluetooth Protocol Stack - Read me file
//  Copyright (C) 2013-2016 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//  Description
//      This is the protocol stack of bluetooth for small embedded device.
//      Supported function:
//          SDP     Service discovery protocol
//          RFCOMM  Serial communication
//          ATT     Attribute protocol
//      Depend reference:
//          time.h              time()
//          rijndael-api-ref.h  AES cryptographic routines
//
//  Usage:
//      Program flow is an event loop type.
//      Example
//          #include <time.h>
//          #include <string.h>
//          #include "bt_rfcomm_server.h" // Bluetooth RFCOMM server access API
//          #include "bt_att_server.h"    // Bluetooth Attribute protocol server access API
//          #include "bt_att_db.h"        // Bluetooth Attribute protocol database access API
//          main(){
//              time_t tm;
//              char  *msg = "Hello world.\n";
//
//                :
//              tm = 0;
//                :
//              for(;;) {
//                    :
//                  if (BSP_SwitchStateGet(BSP_SWITCH_1) == BSP_SWITCH_STATE_PRESSED) {
//                      bt_rfcommServerDataSendRequest(msg, strlen(msg));
//                  }
//                  if (tm != time(NULL)) {
//                      tm  = time(NULL);
//                      bt_attDbUpdate(0x1012, &tm, sizeof(tm)); // and notify
//                  }
//                    :
//              }
//          }
//
//          void bt_rfcommServerDataRecived(unsigned char *inBuff, size_t inLen){
//              printf("RFCOMM recv: %s\n", inBuff);
//              return;
//          }
//
//          void bt_attDbNotification(unsigned short handle, void *value, size_t valueLen){
//              bt_att_serverNotificationQueueAdd(handle);    // Essential action
//              switch(handle) {
//              case 0x1002:
//                  if (*(unsigned char *)value == 0x00) {
//                      BSP_LEDOff(BSP_LED_1);
//                  } else {
//                      BSP_LEDOn(BSP_LED_1);
//                  }
//                  break;
//              default:
//                  break;
//              }
//              return;
//          }
//
//  Reference:
//      Bluetooth SIG (www.bluetooth.com)
//          BLUETOOTH SPECIFICATION Version 4.0
//
//  Device: STM32F4
//      IDE     : Eclipse IDE for C/C++ Developers, Juno Service Release 1
//      Compiler: gcc-arm-none-eabi 4.7
//      Library : STM32F4xx Standard Peripherals Library Drivers (StdPeriph_Driver) V1.0.0
//  Device: PIC32MX
//      IDE     : MPLAB X IDE v2.26
//      Compiler: Xc32-gcc(v1.33)
//      Library : Microchip Harmony V1.01
//
//  Update history
//  ---------- ----- -------------------------------------------
//  2013.01.21 v0.0  First cording
//  2013.04.21 v0.1  Commit
//  2014.03.05 v0.2  Change the USB process calling
//  2015.02.03 v0.3  Supported the PIC32
//  2015.02.06 v0.3a Supported the Secure Simple Pairing
//  2015.02.06 v0.3a SDP: Securely it tried to do search processing (but not support AttributeIDList condition)
//  2015.02.20 v0.3b The new "bt_att_data.c" is independent of the ATT database user data
//  2015.03.03 v0.3b Concealment of bluetooth task to the user
//  2016.02.12 v0.3c Appended a return code, Supported the XC32 v1.40
//  2016.02.15 v0.3c Disabled the Security Manager, default service
//  2016.02.19 v0.3c+ Change of the PIN response(HCI command/event)
//  2016.02.20 v0.3c+ Bug fix, bt_l2cap??????GetNext()
//
