//
//  bt_att_data.c
//
//  Bluetooth Protocol Stack - Attribute Database manager - Database content
//      Watt Hour Meter 0.6.5
//
//  Update history
//  ---------- ----- -------------------------------------------
//  2015.02.20 v0.3b The new "bt_att_data.c" is independent of the ATT database user data
//  2016.01.24 v0.0  WHM 0.6.5
//

#include "bt_spec_gatt.h"
#include "bt_att_db.h"
#include "bt_att_data.h"

#define UINT16(a)  ATT_UINT16_DATA(a)       // Alias (Short name)
#define UINT32(a)  ATT_UINT32_DATA(a)       // Alias
#define UU16(a)    ATT_UUID16TO128_DATA(a)  // Alias
#define UU128(a)   ATT_UUID128_DATA(a)      // Alias

//
//  ATT Database  -  WHM0.6.5
//
struct bt_attDb bt_attDb = {
    77,
    {// Handle  Type                                                                                    Len    Values                                                                                                                           Property
        {0x0001,{UU16(GATT_Attribute_Types_Primary_Service)},                                            2,    {UINT16(GATT_Services_Generic_Access_Profile)},                                                                                  ATT_DB_PROPERTIES_READ},
        {0x0002,    {UU16(GATT_Attribute_Types_Characteristic)},                                         5,    {GATT_Characteristic_Properties_Read, UINT16(0x0003), UINT16(GATT_Characteristic_Types_Device_Name)},                            ATT_DB_PROPERTIES_READ},
        {0x0003,        {UU16(GATT_Characteristic_Types_Device_Name)},                                  14,    {'W', 'H', ' ', 'M', 'e', 't', 'e', 'r', ' ', '0', '.', '6', '.', '5', '\0'},                                                    ATT_DB_PROPERTIES_READ},
        {0x0004,    {UU16(GATT_Attribute_Types_Characteristic)},                                         5,    {GATT_Characteristic_Properties_Read, UINT16(0x0005), UINT16(GATT_Characteristic_Types_Appearance)},                             ATT_DB_PROPERTIES_READ},
        {0x0005,        {UU16(GATT_Characteristic_Types_Appearance)},                                    2,    {UINT16(GATT_Characteristic_Appearance_Category_Generic_Computer)},                                                              ATT_DB_PROPERTIES_READ},

        {0x0011,{UU16(GATT_Attribute_Types_Primary_Service)},                                            2,    {UINT16(GATT_Services_Generic_Attribute_Profile)},                                                                               ATT_DB_PROPERTIES_READ},
        {0x0012,    {UU16(GATT_Attribute_Types_Characteristic)},                                         5,    {GATT_Characteristic_Properties_Read|GATT_Characteristic_Properties_Write_Without_Response|GATT_Characteristic_Properties_Indicate, UINT16(0x0013), UINT16(GATT_Characteristic_Types_Service_Changed)},    ATT_DB_PROPERTIES_READ},
        {0x0013,        {UU16(GATT_Characteristic_Types_Service_Changed)},                               4,    {UINT16(0x0001), UINT16(0xffff)},                                                                                                ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
        {0x0014,        {UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},      2,    {UINT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},                                                          ATT_DB_PROPERTIES_READ},
        {0x0015,        {UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},     2,    {UINT16(0x0000)},                                                                                                                ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
//===============================================================================================================================================================================================================================================================================================
        {0x1000,{UU16(GATT_Attribute_Types_Primary_Service)},                                           16,    {UU128(UUID_MY_SERVICE)},                                                                                                        ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
        {0x1001,    {UU16(GATT_Attribute_Types_Characteristic)},                                        19,    {GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Write | GATT_Characteristic_Properties_Notify, UINT16(0x1002), UU128(UUID_MY_CHAR0)},    ATT_DB_PROPERTIES_READ},
        {0x1002,        {UU128(UUID_MY_CHAR0)},                                                          7,    {UINT16(2016), 1, 24, 12, 34, 56},                                                                                               ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
        {0x1003,        {UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},      2,    {UINT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},                                                          ATT_DB_PROPERTIES_READ},
        {0x1004,        {UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},     2,    {UINT16(0x0000)},                                                                                                                ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
        {0x1005,        {UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},         8,    {'D', 'a', 't', 'e', 'T', 'i', 'm', 'e'},                                                                                        ATT_DB_PROPERTIES_READ},
        {0x1006,        {UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},                   7,    {GATT_Characteristic_Format_struct, 0, UINT16(GATT_Characteristic_Unit_time_second),
                                                                                                                GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, UINT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},    ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
        {0x1011,    {UU16(GATT_Attribute_Types_Characteristic)},                                        19,    {GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Notify, UINT16(0x1012), UU128(UUID_MY_CHAR1)},             ATT_DB_PROPERTIES_READ},
        {0x1012,        {UU128(UUID_MY_CHAR1)},                                                         20,    {UINT32(1),UINT32(2),UINT32(3),UINT32(4),UINT32(5)},                                                                             ATT_DB_PROPERTIES_READ},
        {0x1013,        {UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},      2,    {UINT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},                                                          ATT_DB_PROPERTIES_READ},
        {0x1014,        {UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},     2,    {UINT16(0x0000)},                                                                                                                ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
        {0x1015,        {UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},         8,    {'C', 'H', '1', ' ', 'd', 'a', 't', 'a'},                                                                                        ATT_DB_PROPERTIES_READ},
        {0x1016,        {UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},                   7,    {GATT_Characteristic_Format_float32, 0, UINT16(GATT_Characteristic_Unit_power_watt),
                                                                                                                GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, UINT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},    ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
        {0x1021,    {UU16(GATT_Attribute_Types_Characteristic)},                                        19,    {GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Notify, UINT16(0x1022), UU128(UUID_MY_CHAR2)},             ATT_DB_PROPERTIES_READ},
        {0x1022,        {UU128(UUID_MY_CHAR2)},                                                         20,    {UINT32(1),UINT32(2),UINT32(3),UINT32(4),UINT32(5)},                                                                             ATT_DB_PROPERTIES_READ},
        {0x1023,        {UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},      2,    {UINT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},                                                          ATT_DB_PROPERTIES_READ},
        {0x1024,        {UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},     2,    {UINT16(0x0000)},                                                                                                                ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
        {0x1025,        {UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},         8,    {'C', 'H', '2', ' ', 'd', 'a', 't', 'a'},                                                                                        ATT_DB_PROPERTIES_READ},
        {0x1026,        {UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},                   7,    {GATT_Characteristic_Format_float32, 0, UINT16(GATT_Characteristic_Unit_power_watt),
                                                                                                                GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, UINT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},    ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
        {0x1031,    {UU16(GATT_Attribute_Types_Characteristic)},                                        19,    {GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Notify, UINT16(0x1032), UU128(UUID_MY_CHAR3)},             ATT_DB_PROPERTIES_READ},
        {0x1032,        {UU128(UUID_MY_CHAR3)},                                                         20,    {UINT32(1),UINT32(2),UINT32(3),UINT32(4),UINT32(5)},                                                                             ATT_DB_PROPERTIES_READ},
        {0x1033,        {UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},      2,    {UINT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},                                                          ATT_DB_PROPERTIES_READ},
        {0x1034,        {UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},     2,    {UINT16(0x0000)},                                                                                                                ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
        {0x1035,        {UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},         8,    {'C', 'H', '3', ' ', 'd', 'a', 't', 'a'},                                                                                        ATT_DB_PROPERTIES_READ},
        {0x1036,        {UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},                   7,    {GATT_Characteristic_Format_float32, 0, UINT16(GATT_Characteristic_Unit_power_watt),
                                                                                                                GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, UINT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},    ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
        {0x1041,     {UU16(GATT_Attribute_Types_Characteristic)},                                       19,    {GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Notify, UINT16(0x1042), UU128(UUID_MY_CHAR4)},             ATT_DB_PROPERTIES_READ},
        {0x1042,         {UU128(UUID_MY_CHAR4)},                                                        20,    {UINT32(1),UINT32(2),UINT32(3),UINT32(4),UINT32(5)},                                                                             ATT_DB_PROPERTIES_READ},
        {0x1043,         {UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},     2,    {UINT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},                                                          ATT_DB_PROPERTIES_READ},
        {0x1044,         {UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},    2,    {UINT16(0x0000)},                                                                                                                ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
        {0x1045,         {UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},        8,    {'C', 'H', '4', ' ', 'd', 'a', 't', 'a'},                                                                                        ATT_DB_PROPERTIES_READ},
        {0x1046,         {UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},                  7,    {GATT_Characteristic_Format_float32, 0, UINT16(GATT_Characteristic_Unit_power_watt),
                                                                                                                GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, UINT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},    ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
        {0x1101,     {UU16(GATT_Attribute_Types_Characteristic)},                                       19,    {GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Write | GATT_Characteristic_Properties_Notify, UINT16(0x1102), UU128(UUID_MY_CHAR10)},    ATT_DB_PROPERTIES_READ},
        {0x1102,         {UU128(UUID_MY_CHAR10)},                                                        5,    {UINT16(2016),1,1,0xff},                                                                                                         ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
        {0x1103,         {UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},     2,    {UINT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},                                                          ATT_DB_PROPERTIES_READ},
        {0x1104,         {UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},    2,    {UINT16(0x0000)},                                                                                                                ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
        {0x1105,         {UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},        7,    {'H', 'i', 's', 't', 'o', 'r', 'y'},                                                                                             ATT_DB_PROPERTIES_READ},
        {0x1106,         {UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},                  7,    {GATT_Characteristic_Format_struct, 0x00, UINT16(GATT_Characteristic_Unit_unitless),
                                                                                                                GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, UINT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},    ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
        {0x1111,     {UU16(GATT_Attribute_Types_Characteristic)},                                       19,    {GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Notify, UINT16(0x1112), UU128(UUID_MY_CHAR11)},            ATT_DB_PROPERTIES_READ},
        {0x1112,         {UU128(UUID_MY_CHAR11)},                                                       14,    {UINT16(1),UINT16(2),UINT16(3),UINT16(4),UINT16(5),UINT16(6),UINT16(7)},                                                         ATT_DB_PROPERTIES_READ},
        {0x1113,         {UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},     2,    {UINT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},                                                          ATT_DB_PROPERTIES_READ},
        {0x1114,         {UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},    2,    {UINT16(0x0000)},                                                                                                                ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
        {0x1115,         {UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},        6,    {'A', 'n', 's', 'w', 'e', 'r'},                                                                                                  ATT_DB_PROPERTIES_READ},
        {0x1116,         {UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},                  7,    {GATT_Characteristic_Format_struct, 0, UINT16(GATT_Characteristic_Unit_unitless),
                                                                                                                GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, UINT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},    ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
        {0x1121,     {UU16(GATT_Attribute_Types_Characteristic)},                                       19,    {GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Write | GATT_Characteristic_Properties_Notify, UINT16(0x1122), UU128(UUID_MY_CHAR12)},    ATT_DB_PROPERTIES_READ},
        {0x1122,         {UU128(UUID_MY_CHAR12)},                                                       20,    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},                                                                                       ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
        {0x1123,         {UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},     2,    {UINT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},                                                          ATT_DB_PROPERTIES_READ},
        {0x1124,         {UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},    2,    {UINT16(0x0000)},                                                                                                                ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
        {0x1125,         {UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},        5,    {'S', 'e', 't', 'u', 'p'},                                                                                                       ATT_DB_PROPERTIES_READ},
        {0x1126,         {UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},                  7,    {GATT_Characteristic_Format_struct, 0x00, UINT16(GATT_Characteristic_Unit_unitless),
                                                                                                                GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, UINT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},    ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
        {0x1131,     {UU16(GATT_Attribute_Types_Characteristic)},                                       19,    {GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Notify, UINT16(0x1132), UU128(UUID_MY_CHAR13)},            ATT_DB_PROPERTIES_READ},
        {0x1132,         {UU128(UUID_MY_CHAR13)},                                                       20,    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},                                                                                       ATT_DB_PROPERTIES_READ},
        {0x1133,         {UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},     2,    {UINT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},                                                          ATT_DB_PROPERTIES_READ},
        {0x1134,         {UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},    2,    {UINT16(0x0000)},                                                                                                                ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
        {0x1135,         {UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},        7,    {'S', 'e', 't', 't', 'i', 'n', 'g'},                                                                                             ATT_DB_PROPERTIES_READ},
        {0x1136,         {UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},                  7,    {GATT_Characteristic_Format_struct, 0, UINT16(GATT_Characteristic_Unit_unitless),
                                                                                                                GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, UINT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},    ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
        {0x1141,     {UU16(GATT_Attribute_Types_Characteristic)},                                       19,    {GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Write | GATT_Characteristic_Properties_Notify, UINT16(0x1142), UU128(UUID_MY_CHAR14)},    ATT_DB_PROPERTIES_READ},
        {0x1142,         {UU128(UUID_MY_CHAR14)},                                                        7,    {0,0,UINT16(2016),1,1,0xff},                                                                                                     ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
        {0x1143,         {UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},     2,    {UINT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},                                                          ATT_DB_PROPERTIES_READ},
        {0x1144,         {UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},    2,    {UINT16(0x0000)},                                                                                                                ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
        {0x1145,         {UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},        8,    {'H', 'i', 's', 't', 'o', 'r', 'y', '2'},                                                                                        ATT_DB_PROPERTIES_READ},
        {0x1146,         {UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},                  7,    {GATT_Characteristic_Format_struct, 0x00, UINT16(GATT_Characteristic_Unit_unitless),
                                                                                                                GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, UINT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},    ATT_DB_PROPERTIES_READ},
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
        {0x1151,     {UU16(GATT_Attribute_Types_Characteristic)},                                       19,    {GATT_Characteristic_Properties_Read | GATT_Characteristic_Properties_Notify, UINT16(0x1152), UU128(UUID_MY_CHAR15)},            ATT_DB_PROPERTIES_READ},
        {0x1152,         {UU128(UUID_MY_CHAR15)},                                                        4,    {UINT32(0)},                                                                                                                     ATT_DB_PROPERTIES_READ},
        {0x1153,         {UU16(GATT_Characteristic_Descriptors_Characteristic_Extended_Properties)},     2,    {UINT16(GATT_Characteristic_Extended_Properties_Writable_Auxiliaries)},                                                          ATT_DB_PROPERTIES_READ},
        {0x1154,         {UU16(GATT_Characteristic_Descriptors_Client_Characteristic_Configuration)},    2,    {UINT16(0x0000)},                                                                                                                ATT_DB_PROPERTIES_READ | ATT_DB_PROPERTIES_WRITE},
        {0x1155,         {UU16(GATT_Characteristic_Descriptors_Characteristic_User_Description)},        7,    {'A', 'n', 's', 'w', 'e', 'r', '2'},                                                                                             ATT_DB_PROPERTIES_READ},
        {0x1156,         {UU16(GATT_Characteristic_Descriptors_Characteristic_Format)},                  7,    {GATT_Characteristic_Format_float32, 0, UINT16(GATT_Characteristic_Unit_energy_kilowatt_hour),
                                                                                                                GATT_Characteristic_Namespace_Bluetooth_SIG_Assigned_Numbers, UINT16(GATT_Characteristic_Description_Bluetooth_SIG_inside)},    ATT_DB_PROPERTIES_READ},
    }
};
