//
//  bt_l2cap_cb_channel.h
//
//  Bluetooth Protocol Stack - Logical channel management control block operation
//  Copyright (C) 2013-2015 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//  Update history
//  ---------- ----- -------------------------------------------
//  2013.01.21 v0.0  First cording
//  2013.04.21 v0.1  Commit
//  2013.05.08 v0.1  Changed a control block structure, bt_l2capChannelGetNext argument
//  2015.02.03 v0.3  Supported the PIC32
//
#ifndef L2CAP_CHANNEL_H_
#define L2CAP_CHANNEL_H_

#include <string.h>
#include "bt_l2cap_cb_link.h"

//
//  Control block
//
struct bt_l2capChannel {
    struct bt_l2capChannel *next;
    struct bt_l2capLink    *link;       // handle,...
    unsigned short          scid;
    unsigned short          dcid;
    unsigned short          psm;
    size_t                  (*serviceProg)(unsigned short handle, unsigned char *inBuff, size_t inLen, unsigned char *outBuff, size_t outSize);
    enum {
        CLOSED = 0,
        WAIT_CONNECT,
        WAIT_CONFIG,
        WAIT_CONFIG_REQ_RSP,
        WAIT_CONFIG_REQ,
        WAIT_SEND_CONFIG,
        WAIT_CONFIG_RSP,
        OPEN,
        WAIT_DISCONNECT,
        WAIT_CONNECT_RSP
    }                       state;
    enum {
        NONE = 0,
        DATA_RECIEVE,
        SEND_REQUEST,
        DISCONNECT_REQUEST
    }                       event;
    long                    try;        // Timeout counter
    unsigned char           id;         // Message identifier counter

    unsigned char           userArea[8];// Application work
};

//
//  API
//
extern struct bt_l2capChannel  *bt_l2capChannelGetNext(struct bt_l2capChannel **id);
extern struct bt_l2capChannel  *bt_l2capChannelFind(unsigned short handle, unsigned short scid, unsigned short dcid);
extern struct bt_l2capChannel  *bt_l2capChannelAdd(unsigned short handle, unsigned short scid, unsigned short dcid);
extern void                     bt_l2capChannelDelete(struct bt_l2capChannel *p);
extern void                     bt_l2capChannelClear(void);                         // ADD 2015.02.03

#endif /* L2CAP_CHANNEL_H_ */
