//
//  bt_l2cap_cb_device.h
//
//  Bluetooth Protocol Stack - Remote BT controller device management control block operation
//  Copyright (C) 2013-2015 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//  Update history
//  ---------- ----- -------------------------------------------
//  2013.01.21 v0.0  First cording
//  2013.04.21 v0.1  Commit
//  2013.05.08 v0.1  Changed a control block structure
//  2015.02.06 v0.3a Supported the Secure Simple Pairing
//
#ifndef BT_L2CAP_CB_DEVICE_H_
#define BT_L2CAP_CB_DEVICE_H_

//
//  Control block
//
struct bt_l2capDevice {
    struct bt_l2capDevice  *next;
    unsigned char           bdAddr[6];
    unsigned char           linkKey[16];
    unsigned char           keyType;
    unsigned char           peerAddressType;    // LE
//  unsigned char           peerAddress[6];     // LE  -> bdAddr
    unsigned char           ioCapability;       // ADD 2015.02.06
    unsigned char           oobDataPresent;     // ADD 2015.02.06
    unsigned char           authentication;     // ADD 2015.02.06
    unsigned long           numericValue;       // ADD 2015.02.06

    unsigned char           bdAddrValid;
    unsigned char           linkKeyValid;
    unsigned char           sspValid;           // ADD 2015.02.06  Secure Simple Pairing
    char                   *name;
};

//
//  API
//
extern struct bt_l2capDevice   *bt_l2capDeviceGetNext(struct bt_l2capDevice **id);  // ADD 2015.02.03
extern struct bt_l2capDevice   *bt_l2capDeviceFind(unsigned char *bdAddr);
extern struct bt_l2capDevice   *bt_l2capDeviceAdd(unsigned char *bdAddr);
extern void                     bt_l2capDeviceDelete(struct bt_l2capDevice *p);
extern void                     bt_l2capDeviceClear(void);                          // ADD 2015.02.03

#endif /* BT_L2CAP_CB_DEVICE_H_ */
