//
//  flash.h
//
//  Internal FLASH memory Run-TimeSelf-Programming(RTSP)
//  Copyright(C)2016 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    example:
//          volatile const char xyz[2048] __attribute__((space(prog),aligned(FLASH_BYTE_PAGE_SIZE)));
//
//          if (FLASH_Erase(xyz, sizeof(xyz) != 0) {
//              printf("ERROR\n");
//          } else {
//              int abc;    // word boundary area
//              int i;
//
//              for(i = 0; i < 10; i++) {
//                  memset(&abc, 0x55, sizeof(abc));
//                  if (FLASH_Write(&xyz[i*4], &abc, sizeof(abc) != 0) {
//                      printf("ERROR\n");
//                      break;
//                  }
//              }
//          }
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2016.01.29 v0.0  First cording
//
//
#ifndef FLASH_H_
#define FLASH_H_
#include <stddef.h>

//
//  Config
//
#define FLASH_BYTE_PAGE_SIZE  1024          // PIC32MX1xx/2xx
#define FLASH_BYTE_ROW_SIZE   128           // PIC32MX1xx/2xx
#define FLASH_BYTE_WORD_SIZE  sizeof(int)   // PIC32MX1xx/2xx

#define FLASH_DMA_ID    DMA_ID_0
#define FLASH_NVM_ID    NVM_ID_0

//
//  API
//
extern int  FLASH_Erase(void *dest, size_t len);
// dest: flash address, page boundary
// len : Byte, auto round up a page
extern int  FLASH_Write(void *dest, void *src, size_t len);
// dest: flash address, word boundary
// src : RAM address, word boundary
// len : Byte, auto round up a word

#endif
