//
//  ic_led.h
//
//  IC LED (WS2812B: Intelligent control LED) support
//  Copyright(C)2015-2016 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2015.05.08 v0.0  First cording
//    2016.01.22 v0.0  Appended a color name
//
//
#ifndef IC_LED_H_
#define IC_LED_H_

//
//  Config
//
#define ICLED_NUM       4           // RGB LED string number
#define ICLED_BRIGHT    255         // Physical bright 1~255
#define ICLED_DMA       DMA_ID_0    // Undefine is softwere polling
#define ICLED_SPI       SPI_ID_1

//
//  Constant value
//
#define ICLED_COLOR_BLACK           0.00, 0.00, 0.00
#define ICLED_COLOR_LAMPBLACK       0.14, 0.08, 0.05
#define ICLED_COLOR_DIMGRAY         0.41, 0.41, 0.41
#define ICLED_COLOR_GRAY            0.50, 0.50, 0.50
#define ICLED_COLOR_DARKGRAY        0.66, 0.66, 0.66
#define ICLED_COLOR_LIGHTGRAY       0.83, 0.83, 0.83
#define ICLED_COLOR_WHITE           1.00, 1.00, 1.00
#define ICLED_COLOR_IVORY           1.00, 1.00, 0.94
#define ICLED_COLOR_LAVENDER        0.90, 0.90, 0.98
#define ICLED_COLOR_STEELBLUE       0.27, 0.51, 0.71
#define ICLED_COLOR_DARKBLUE        0.00, 0.00, 0.55
#define ICLED_COLOR_MIDNIGHTBLUE    0.10, 0.10, 0.44
#define ICLED_COLOR_BLUE            0.00, 0.00, 1.00
#define ICLED_COLOR_ROYALBLUE       0.25, 0.41, 0.88
#define ICLED_COLOR_SKYBLUE         0.53, 0.81, 0.92
#define ICLED_COLOR_MERMAIDBLUE     0.65, 0.86, 0.89
#define ICLED_COLOR_LIGHTBLUE       0.68, 0.85, 0.90
#define ICLED_COLOR_CYAN            0.00, 1.00, 1.00
#define ICLED_COLOR_LIGHTCYAN       0.88, 1.00, 1.00
#define ICLED_COLOR_DARKCYAN        0.00, 0.55, 0.55
#define ICLED_COLOR_BLUEGREEN       0.00, 0.64, 0.30
#define ICLED_COLOR_DARKGREEN       0.00, 0.39, 0.00
#define ICLED_COLOR_GREEN           0.00, 0.50, 0.00
#define ICLED_COLOR_FORESTGREEN     0.13, 0.55, 0.13
#define ICLED_COLOR_SEAGREEN        0.18, 0.55, 0.34
#define ICLED_COLOR_LIGHTGREEN      0.56, 0.93, 0.56
#define ICLED_COLOR_GREENYELLOW     0.68, 1.00, 0.18
#define ICLED_COLOR_LIME            0.00, 1.00, 0.00
#define ICLED_COLOR_YELLOWGREEN     0.60, 0.80, 0.20
#define ICLED_COLOR_OLIVEDRAB       0.42, 0.56, 0.14
#define ICLED_COLOR_OLIVE           0.50, 0.50, 0.00
#define ICLED_COLOR_TAN             0.82, 0.71, 0.55
#define ICLED_COLOR_KHAKI           0.94, 0.90, 0.55
#define ICLED_COLOR_YELLOW          1.00, 1.00, 0.00
#define ICLED_COLOR_GOLD            1.00, 0.84, 0.00
#define ICLED_COLOR_ORANGE          1.00, 0.65, 0.00
#define ICLED_COLOR_CHOCOLATE       0.82, 0.41, 0.12
#define ICLED_COLOR_DARKRED         0.55, 0.00, 0.00
#define ICLED_COLOR_BROWN           0.65, 0.16, 0.16
#define ICLED_COLOR_SALMON          0.98, 0.50, 0.03
#define ICLED_COLOR_CORAL           1.00, 0.50, 0.31
#define ICLED_COLOR_TOMATO          1.00, 0.39, 0.28
#define ICLED_COLOR_RED             1.00, 0.00, 0.00
#define ICLED_COLOR_DEEPPINK        1.00, 0.08, 0.58
#define ICLED_COLOR_PINK            1.00, 0.75, 0.80
#define ICLED_COLOR_MAGENTA         1.00, 0.00, 1.00
#define ICLED_COLOR_VIOLET          0.93, 0.51, 0.93
#define ICLED_COLOR_PURPLE          0.50, 0.00, 0.50
#define ICLED_COLOR_INDIGO          0.29, 0.00, 0.51
#define ICLED_COLOR_SLATEBLUE       0.42, 0.35, 0.80

//
//  API
//
extern void ICLED_SetColor(int pos, float red, float green, float blue);
extern void ICLED_Init(void);
extern void ICLED_Tasks(void);      // DMA mode is not use

#endif
