//
//  measure.h
//
//  WHM0.6.5 A/D capture and electric power calculation task
//  Copyright(C)2016 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2016.01.17 v0.0  First cording
//    2016.02.28 v0.6.5 Release
//
//
#ifndef MEASURE_H_
#define MEASURE_H_

//
//  Config
//
#define MEASURE_ADC_ID           ADC_ID_1
#define MEASURE_ADC_SRC          INT_SOURCE_ADC_1
#define MEASURE_ADC_VEC          INT_VECTOR_AD1
#define MEASURE_ADC_ISR          _ADC_VECTOR
#define MEASURE_ADC_PRI          INT_PRIORITY_LEVEL3
#define MEASURE_ADC_SUB          INT_SUBPRIORITY_LEVEL1
#define MEASURE_ADC_IPL          ipl3auto

#define MEASURE_TMR_ID           TMR_ID_4
#define MEASURE_TMR_SRC          INT_SOURCE_TIMER_4
#define MEASURE_TMR_VEC          INT_VECTOR_T4
#define MEASURE_TMR_ISR          _TIMER_4_VECTOR
#define MEASURE_TMR_PRI          INT_PRIORITY_LEVEL3
#define MEASURE_TMR_SUB          INT_SUBPRIORITY_LEVEL0
#define MEASURE_TMR_IPL          ipl3auto

#define MEASURE_SAMPLE_INTERVAL  0.0002     // [s]
#define MEASURE_SAMPLE_COUNT     500        //
#define MEASURE_TAD              (200e-9)   // [s] 200[ns] ADC clock period
#define MEASURE_ACQUISITION_TIME (2.0e-6)   // [s] 2[us]
#define MEASURE_VREF             3.3        // [V]
#define MEASURE_RESOLUTION       1024       //
#define MEASURE_UNDER_FLOW       (int)(MEASURE_RESOLUTION * 0.05)
#define MEASURE_OVER_FLOW        (int)(MEASURE_RESOLUTION * 0.95)
#define MEASURE_V_K              ((1100+10)/10.0)    // 1100[k ohm] + 10[k ohm]
#define MEASURE_I1_K             (1 / 0.005 / 11)    // 5[m ohm] x OpAmp gain
#define MEASURE_I10_K            (1 / 0.005 / 110)   //    I=V/R/gain
#define MEASURE_I100_K           (1 / 0.005 / 1100)  //
#define MEASURE_AUTO_OFFSET      1000       // [Count]


//
//  API
//
extern void     MEASURE_Init(void);
extern void     MEASURE_Tasks(void);
extern float    MEASURE_GetPower(int ch);
extern float    MEASURE_GetVoltage(int ch);
extern float    MEASURE_GetCurrent(int ch);
extern float    MEASURE_GetFrequency(int ch);
extern void     MEASURE_GetWaveRequest(short data[][5], int len, int ch);
extern int      MEASURE_GetWaveStatus(void);

#endif
