/*******************************************************************************
  SYS PORTS Static Functions for PORTS System Service

  Company:
    Microchip Technology Inc.

  File Name:
    sys_ports_static.c

  Summary:
    SYS PORTS static function implementations for the Ports System Service.

  Description:
    The Ports System Service provides a simple interface to manage the ports
    on Microchip microcontrollers. This file defines the static implementation for the 
    Ports System Service.
    
  Remarks:
    Static functions incorporate all system ports configuration settings as
    determined by the user via the Microchip Harmony Configurator GUI.  It provides 
    static version of the routines, eliminating the need for an object ID or 
    object handle.

*******************************************************************************/

//DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*******************************************************************************/
//DOM-IGNORE-END

#include "system_config.h"
#include "peripheral/ports/plib_ports.h"
#include "peripheral/int/plib_int.h"
#include "peripheral/devcon/plib_devcon.h"


void SYS_PORTS_Initialize(void)
{
#ifdef DEL
    /* PORT A Initialization */
    PLIB_PORTS_OpenDrainEnable(PORTS_ID_0, PORT_CHANNEL_A, SYS_PORT_A_ODC);
    PLIB_PORTS_Write( PORTS_ID_0, PORT_CHANNEL_A,  SYS_PORT_A_LAT);
    PLIB_PORTS_DirectionOutputSet( PORTS_ID_0, PORT_CHANNEL_A,  SYS_PORT_A_TRIS ^ 0xFFFF);
    PLIB_PORTS_ChangeNoticePerPortTurnOn(PORTS_ID_0, PORT_CHANNEL_A);
    PLIB_PORTS_ChannelModeSelect(PORTS_ID_0, PORT_CHANNEL_A, SYS_PORT_A_ANSEL ^ 0xFFFF, PORTS_PIN_MODE_DIGITAL);
    PLIB_PORTS_ChannelChangeNoticeEnable(PORTS_ID_0, PORT_CHANNEL_A, SYS_PORT_A_CNEN);
    PLIB_PORTS_ChannelChangeNoticePullUpEnable(PORTS_ID_0, PORT_CHANNEL_A, SYS_PORT_A_CNPU);
    PLIB_PORTS_ChannelChangeNoticePullDownEnable(PORTS_ID_0, PORT_CHANNEL_A, SYS_PORT_A_CNPD);
    
    /* PORT B Initialization */
    PLIB_PORTS_OpenDrainEnable(PORTS_ID_0, PORT_CHANNEL_B, SYS_PORT_B_ODC);
    PLIB_PORTS_Write( PORTS_ID_0, PORT_CHANNEL_B,  SYS_PORT_B_LAT);
    PLIB_PORTS_DirectionOutputSet( PORTS_ID_0, PORT_CHANNEL_B,  SYS_PORT_B_TRIS ^ 0xFFFF);
    PLIB_PORTS_ChangeNoticePerPortTurnOn(PORTS_ID_0, PORT_CHANNEL_B);
    PLIB_PORTS_ChannelModeSelect(PORTS_ID_0, PORT_CHANNEL_B, SYS_PORT_B_ANSEL ^ 0xFFFF, PORTS_PIN_MODE_DIGITAL);
    PLIB_PORTS_ChannelChangeNoticeEnable(PORTS_ID_0, PORT_CHANNEL_B, SYS_PORT_B_CNEN);
    PLIB_PORTS_ChannelChangeNoticePullUpEnable(PORTS_ID_0, PORT_CHANNEL_B, SYS_PORT_B_CNPU);
    PLIB_PORTS_ChannelChangeNoticePullDownEnable(PORTS_ID_0, PORT_CHANNEL_B, SYS_PORT_B_CNPD);


    /* PPS Input Remapping */

    /* PPS Output Remapping */

#endif

    // Input or Output                                                                  PORT  Remap  Anlg  ICSP    Peripheral                          Application
                                                                                    //  ----  -----  ----  ------  ----------------------------------  ------------------
    PLIB_PORTS_PinDirectionInputSet (PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_0);  //  RA0   RPA0  <AN0>  PGED3   VREF+,CVREF+,C3INC,CTED1,PMD7       Vsense
    PLIB_PORTS_PinDirectionInputSet (PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_1);  //  RA1   RPA1  <AN1>  PGEC3   VREF-,CVREF-,CTED2,PMD6             AGND
//  PLIB_PORTS_PinDirectionOutputSet(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_2);  //  RA2   RPA2                <OSC1>,CLKI                          Crystal 12MHz
//  PLIB_PORTS_PinDirectionOutputSet(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_3);  //  RA3   RPA3                <OSC2>,CLKO,PMA0                     Crystal 12MHz
    PLIB_PORTS_PinDirectionOutputSet(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_4);  // <RA4>  RPA4                 SOSCO,T1CK,CTED9,PMA1               -
//  PLIB_PORTS_PinDirectionInputSet (PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_0);  //  RB0   RPB0   AN2  <PGED1>  C1IND,C2INB,C3IND,PMD0              ICSP
//  PLIB_PORTS_PinDirectionInputSet (PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_1);  //  RB1   RPB1   AN3  <PGEC1>  C1INC,C2INA,CTED12,PMD1             ICSP
    PLIB_PORTS_PinDirectionInputSet (PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_2);  //  RB2   RPB2  <AN4>          C1INB,C2IND,SDA2,CTED13,PMD2        Isense x 100
    PLIB_PORTS_PinDirectionOutputSet(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_3);  // <RB3>  RPB3   AN5           C1INA,C2INC,RTCC,SCL2,PMWR          -
    PLIB_PORTS_PinDirectionOutputSet(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_4);  // <RB4>  RPB4                 SOSCI                               -
    PLIB_PORTS_PinDirectionOutputSet(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_5);  //  RB5  <RPB5>        TMS     USBID                               OC#2(Beep)
    PLIB_PORTS_PinDirectionOutputSet(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_7);  // <RB7>  RPB7         TDI     CTED3,PMD5,INT0                     A
    PLIB_PORTS_PinDirectionOutputSet(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_8);  //  RB8  <RPB8>        TCK     SCL1,CTED10,PMD4                    SPI#1-SDO(LED DATA)
    PLIB_PORTS_PinDirectionOutputSet(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_9);  // <RB9>  RPB9         TDO     SDA1,CTED4,PMD3                     B
//  PLIB_PORTS_PinDirectionOutputSet(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_10); //  RB10  RPB10        PGED2  <D+>,CTED11                          USB D+
//  PLIB_PORTS_PinDirectionOutputSet(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_11); //  RB11  RPB11        PGEC2  <D->                                 USB D-
    PLIB_PORTS_PinDirectionInputSet (PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_13); //  RB13  RPB13 <AN11>         CTPLS,PMRD                          Isense x 10
    PLIB_PORTS_PinDirectionOutputSet(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_14); //  RB14  RPB14  AN10          CVREFOUT,C3INB,VBUSON,<SCK1>,CTED5  SPI#1-SCK(unuse)
    PLIB_PORTS_PinDirectionInputSet (PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_15); //  RB15  RPB15 <AN9>          C3INA,SCK2,CTED6,PMCS1              Isense x 1

    // Analog or Digital
    PLIB_PORTS_PinModePerPortSelect(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_0,  PORTS_PIN_MODE_ANALOG);    // AN0
    PLIB_PORTS_PinModePerPortSelect(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_1,  PORTS_PIN_MODE_ANALOG);    // AN1
    PLIB_PORTS_PinModePerPortSelect(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_0,  PORTS_PIN_MODE_DIGITAL);   // AN2
    PLIB_PORTS_PinModePerPortSelect(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_1,  PORTS_PIN_MODE_DIGITAL);   // AN3
    PLIB_PORTS_PinModePerPortSelect(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_2,  PORTS_PIN_MODE_ANALOG);    // AN4
    PLIB_PORTS_PinModePerPortSelect(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_3,  PORTS_PIN_MODE_DIGITAL);   // AN5
    PLIB_PORTS_PinModePerPortSelect(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_15, PORTS_PIN_MODE_ANALOG);    // AN9
    PLIB_PORTS_PinModePerPortSelect(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_14, PORTS_PIN_MODE_DIGITAL);   // AN10
    PLIB_PORTS_PinModePerPortSelect(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_13, PORTS_PIN_MODE_ANALOG);    // AN11

    // Open drain
    PLIB_PORTS_PinOpenDrainEnable(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_5);     // RB5(beep) is 5V sink
    PLIB_PORTS_PinWrite          (PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_5, 1);  // RB5 open

    // Peripheral function <--> Pin connect
    PLIB_DEVCON_SystemUnlock(DEVCON_ID_0);
    PLIB_DEVCON_DeviceRegistersUnlock(DEVCON_ID_0, DEVCON_PPS_REGISTERS);
//  PLIB_PORTS_RemapInput (PORTS_ID_0, INPUT_FUNC_xxx,   INPUT_PIN_RPxx);
//  PLIB_PORTS_RemapOutput(PORTS_ID_0, OUTPUT_FUNC_xxx,  INPUT_PIN_RPxx);
    PLIB_PORTS_RemapOutput(PORTS_ID_0, OUTPUT_FUNC_SDO1, OUTPUT_PIN_RPB8);  // SPI#1-SDO
    PLIB_PORTS_RemapOutput(PORTS_ID_0, OUTPUT_FUNC_OC2,  OUTPUT_PIN_RPB5);  // OC#2
    PLIB_DEVCON_DeviceRegistersLock(DEVCON_ID_0, DEVCON_PPS_REGISTERS);
    PLIB_DEVCON_SystemLock(DEVCON_ID_0);

    // Initial level
    PLIB_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_7, 0);            // LED turn off
    PLIB_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_8, 0);            // LED turn off
    PLIB_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_9, 0);            // LED turn off

    return;
}


/*******************************************************************************
 End of File
*/
