//
//  usb_host_bluetooth.h
//
//  USB Host Protocol Stack - USB Wireless controller class for bluetooth dongle
//  for Microchip MPLAB harmony
//  Copyright (C) 2015-2016 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//  Reference:
//      Bluetooth SIG (www.bluetooth.com)
//          BLUETOOTH SPECIFICATION Version 4.0
//              [Vol 4] Host Controller Interface [Transport Layer]
//              Part B: USB Transport Layer
//      Microchip
//          MPLAB Harmony Integrated Software Framework v1.01
//              Help document "help_harmony.pdf", Section "USB Host Library"
//
//  Device: PIC32MX
//      IDE     : MPLAB X IDE v2.26
//      Compiler: Xc32-gcc(v1.33)
//      Library : Microchip Harmony V1.01
//
//  Update history
//  ---------- ----- -------------------------------------------
//  2015.01.29 v0.0  First cording
//  2015.02.03 v0.3  Commit
//  2015.03.03 v0.3b Concealment of bluetooth task to the user
//  2016.02.06 v0.3c Harmony v1.06.02
//
#ifndef _USB_HOST_BLUETOOTH_H_
#define _USB_HOST_BLUETOOTH_H_

#include "usb/usb_host.h"
#ifdef USB_HOST_INDEX_0                 // v1.06 ~
#include "usb/src/usb_host_local.h"
#else
#include "usb/usb_host_client_driver.h"
#endif

#define USB_HOST_BLUETOOTH_CLASS      0xe0    // Wireless controler class
#define USB_HOST_BLUETOOTH_SUBCLASS   1       // RF Controler
#define USB_HOST_BLUETOOTH_PROTOCOL   1       // Bluetooth Programing Interface (HCI)

//
//  USB host class driver jump table
//
#ifdef USB_HOST_INDEX_0                 // v1.06 ~
extern USB_HOST_CLASS_DRIVER usbHostClassDriverBluetooth;
#else
extern USB_HOST_CLIENT_DRIVER           gUSBHostBluetoothClientDriver;
#define USB_HOST_BLUETOOTH_INTERFACE    &gUSBHostBluetoothClientDriver
#endif

//
//  Handler
//
typedef uintptr_t USB_HOST_BLUETOOTH_HANDLE;
typedef uintptr_t USB_HOST_BLUETOOTH_INDEX;
typedef enum {
    USB_HOST_BLUETOOTH_EVENT_ATTACH,
    USB_HOST_BLUETOOTH_EVENT_DETACH,
    USB_HOST_BLUETOOTH_EVENT_ERROR,
} USB_HOST_BLUETOOTH_EVENT;

typedef bool (*USB_HOST_BLUETOOTH_EVENT_HANDLER)(
    USB_HOST_BLUETOOTH_INDEX    index,
    USB_HOST_BLUETOOTH_EVENT    event,
    void                        *pData
);

USB_ERROR USB_HOST_Bluetooth_EventHandlerSet(USB_HOST_BLUETOOTH_EVENT_HANDLER handler);

#define USB_HOST_BLUETOOTH_EventHandlerSet(a)   USB_HOST_Bluetooth_EventHandlerSet(a)   // Alias

#endif
